#ifndef H_VECTOR
#define H_VECTOR

#ifndef H_VPF
#include "vpf.h"
#endif

#ifndef H_MACHINE
#include "machine.h"
#endif

#ifndef MAGIC_VECTOR
#define MAGIC_VECTOR   110000000L
#endif

#ifndef MAGIC_VEC_DATA /*SR 10/02*/
#define MAGIC_VEC_DATA 120000001L
#endif

/*SR 10/02*/
/***************************************************************
@    Vec_Data flags
****************************************************************
*/
#define VEC_DATA_VALID_MASK              0x0001
#define VEC_DATA_NEEDS_REDRAW_MASK       0x0002
/*
Description:
*/

/**************************************************************************/
/* VECSIZE                                                                */
/**************************************************************************/
typedef struct
   {
   long magic;
   TABLE_DATA   dht;
   TABLE_DATA   lat;
   TABLE_DATA   cat;
   TABLE_DATA   grt;
   TABLE_DATA   lht;
   TABLE_DATA   edg;
   TABLE_DATA   edx;
   TABLE_DATA   txt;
   TABLE_DATA   txx;
   TABLE_DATA   end;
   TABLE_DATA   enx;
   } VECSIZE;


/* SR 09/16 */
/***************************************************************
@    VEC_DATA
****************************************************************
Defines a vector data structure
*/
typedef struct
{
    long magic;
    long bit_flags;
    BOOLEAN contains_valid_data;
    Pfloat edge_width;
    Pint   edge_color;
    Pint   text_color;
    Pint   point_color;
    VECTOR *vector;
    FILE_SPEC file_spec;

} VEC_DATA;
/*
Description:
This is the structure pointed to by product_pointer in OVERLAYS when the
overlay type is VECT
*/


/* XVT_CC_PROTOtype Definitions */
#if XVT_CC_PROTO
   ERRSTATUS vector_construct (FILE*, VECTOR**, BYTE_ORDER *bo);
   ERRSTATUS vector_destruct (FILE *file, BOOLEAN destruct, VECTOR**,BYTE_ORDER*);
   ERRSTATUS vector_write (FILE *file, VECTOR*,BYTE_ORDER*);
   ERRSTATUS dht_construct (DHT**);
   ERRSTATUS lat_construct (LAT**);
   ERRSTATUS cat_construct (CAT**);
   ERRSTATUS grt_construct (GRT**);
   ERRSTATUS lht_construct (LHT**);
   ERRSTATUS idx_construct (IDX**);
   ERRSTATUS edg_construct (EDG**);
   ERRSTATUS txt_construct (TXT**);
   ERRSTATUS end_construct (END**);
   ERRSTATUS ft_construct  (FT**);
   ERRSTATUS dht_destruct (DHT**);
   ERRSTATUS lat_destruct (LAT**);
   ERRSTATUS cat_destruct (CAT**);
   ERRSTATUS grt_destruct (GRT**);
   ERRSTATUS lht_destruct (LHT**);
   ERRSTATUS idx_destruct (IDX**);
   ERRSTATUS edg_destruct (EDG**);
   ERRSTATUS txt_destruct (TXT**);
   ERRSTATUS end_destruct (END**);
   ERRSTATUS ft_destruct  (FT**);
   ERRSTATUS dht_header (VPT**);
   ERRSTATUS lat_header (VPT**);
   ERRSTATUS cat_header (VPT**);
   ERRSTATUS grt_header (VPT**);
   ERRSTATUS lht_header (VPT**);
   ERRSTATUS edg_header (VPT**);
   ERRSTATUS txt_header (VPT**);
   ERRSTATUS end_header (VPT**);
   long header_size(VPT *);
#else
   ERRSTATUS vector_write ();
   ERRSTATUS vector_construct();
   ERRSTATUS vector_destruct ();
   ERRSTATUS dht_construct ();
   ERRSTATUS lat_construct ();
   ERRSTATUS cat_construct ();
   ERRSTATUS grt_construct ();
   ERRSTATUS lht_construct ();
   ERRSTATUS idx_construct ();
   ERRSTATUS edg_construct ();
   ERRSTATUS txt_construct ();
   ERRSTATUS end_construct ();
   ERRSTATUS ft_construct  ();
   ERRSTATUS dht_destruct ();
   ERRSTATUS lat_destruct ();
   ERRSTATUS cat_destruct ();
   ERRSTATUS grt_destruct ();
   ERRSTATUS lht_destruct ();
   ERRSTATUS idx_destruct ();
   ERRSTATUS edg_destruct ();
   ERRSTATUS txt_destruct ();
   ERRSTATUS end_destruct ();
   ERRSTATUS ft_destruct  ();
   ERRSTATUS dht_header ();
   ERRSTATUS lat_header ();
   ERRSTATUS cat_header ();
   ERRSTATUS grt_header ();
   ERRSTATUS lht_header ();
   ERRSTATUS end_header ();
   ERRSTATUS txt_header ();
   ERRSTATUS edg_header ();
   long header_size();
#endif


/* SR 09/25--list of functions for constructing map versions with different
   vec_data structures */
#if XVT_CC_PROTO
ERRSTATUS vec_con_vers0(FILE *file, VEC_DATA **pointer , BYTE_ORDER *bo);
#else
ERRSTATUS vec_con_vers0();
#endif
/*   Memory allocation and initialization for version 0 maps whose
 *   product_pointers point to a VECTOR structure. Only called when opening
 *   an old .map file: added overlays will be created with vec_data_construct
 *   for the latest version of maps.*/


#if XVT_CC_PROTO
ERRSTATUS vec_con_vers1(FILE *file, VEC_DATA **pointer, BYTE_ORDER *bo);
#else
ERRSTATUS vec_con_vers1();
#endif
/*   Memory allocation and initialization of the VEC_DATA structure in
 *   version 1 maps, which contains edge color and width, text and point
 *   color, and a pointer to a VECTOR structure. */


/***************************************************************
@     (*vec_data_construct[])()
****************************************************************
list of functions to construct vec_data structure for each map version
*/
#if XVT_CC_PROTO
extern ERRSTATUS (*vec_data_construct[2])(FILE *file, VEC_DATA **pointer, BYTE_ORDER *bo);
#else
extern ERRSTATUS (*vec_data_construct[2])();
#endif


/***************************************************************
@     vec_data_destruct()
****************************************************************
function to destruct vec_data structure 
*/
#if XVT_CC_PROTO
ERRSTATUS vec_data_destruct(FILE *file,BOOLEAN destruct,VEC_DATA **vec_data,BYTE_ORDER *bo);
#else
ERRSTATUS vec_data_destruct();
#endif


/************************************************************************
@     decode_vec_data()
*************************************************************************
function to decode the vec_data structure
*/
#if XVT_CC_PROTO
ERRSTATUS decode_vec_data (unsigned char *record, VEC_DATA *vec_data);
#else
ERRSTATUS decode_vec_data();
#endif

/************************************************************************
@     encode_vec_data()
*************************************************************************
A function to encode the vec_data structure*/
#if XVT_CC_PROTO
ERRSTATUS encode_vec_data (unsigned char *record, VEC_DATA *vec_data);
#else
ERRSTATUS encode_vec_data();
#endif


#endif /* H_VECTOR */
