/*
 * Copyright 1995-1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

/*
 * MapCatalog implementation.
 */

#include <MapUnidraw/mapcatalog.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapcreator.h>
#include <MapUnidraw/maproute.h>
#include <OverlayUnidraw/ovfile.h>
#include <Unidraw/unidraw.h>
#include <Attribute/paramlist.h>
#include <stdio.h>
#include <stream.h>
#include <ctype.h>

/*****************************************************************************/

MapCatalog::MapCatalog (
    const char* name, Creator* creator
) : OverlayCatalog(name, creator) {
}

boolean MapCatalog::Retrieve (const char* name, Component*& comp) {
    OverlayCatalog* oldinst = OverlayCatalog::Instance();
    OverlayCatalog::Instance(this);

    if (Valid(name, comp)) {
        _valid = true;

    } else if (UnidrawFormat(name)) {
        _valid = Catalog::Retrieve(name, comp);

    } else {
        filebuf fbuf;
        _valid = fbuf.open(name, input) != 0;

        if (_valid) {
	    istream in(&fbuf);
	    const int len = BUFSIZ;
	    char buf[len];

	    char ch;
	    while (isspace(ch = in.get())); in.putback(ch);
	    ParamList::parse_token(in, buf, len);
	    if (strcmp(buf, "mapfcl") == 0) { 
		comp = new MapFClassComp(in, _parent);
		_valid = in.good() && ((OverlayComp*)comp)->valid();
	    } else if (strcmp(buf, "mapserv") == 0 || 
		       strcmp(buf, "flipbook") == 0) { 
		comp = new MapIdrawComp(in, name);
		_valid = in.good() && ((OverlayComp*)comp)->valid();
	    } else 
		_valid = false;

            if (_valid) {
                Forget(comp, name);
                Register(comp, name);
            }
        }
    }
    OverlayCatalog::Instance(oldinst);
    return _valid;
}

OverlayComp* MapCatalog::ReadComp(const char* name, istream& in, OverlayComp* parent) {
  OverlayComp* child = nil;

  if (strcmp(name, "point") == 0 )
    child = new MapFeatureComp(in, MapFeature::PointType, parent);
  else if (strcmp(name, "line") == 0 )          
    child = new MapFeatureComp(in, MapFeature::LineType, parent);
  else if (strcmp(name, "area") == 0 )
    child = new MapFeatureComp(in, MapFeature::AreaType, parent);
  else if (strcmp(name, "text") == 0 )
    child = new MapFeatureComp(in, MapFeature::TextType, parent);
  else if (strcmp(name, "junction") == 0 )
    child = new MapFeatureComp(in, MapFeature::JunctionType, parent);
  else if (strcmp(name, "route") == 0 )
    child = new MapRouteComp(in, parent);
  else if (strcmp(name, "mapfcl") == 0 )
      child = new MapFClassComp(in, parent);
  else if (strcmp(name, "drawtool") == 0 )
      child = new OverlayFileComp(in, parent);
  else 
    child = OverlayCatalog::ReadComp(name, in, parent);
  return child;
}

MapCatalog* MapCatalog::_instance = nil;

MapCatalog* MapCatalog::Instance() {
  if (!_instance) {
    if (Component::use_unidraw()) 
      _instance = (MapCatalog*)unidraw->GetCatalog();
    else
      _instance = new MapCatalog("MapCatalog", new MapCreator());
  }
  return _instance;
}

void MapCatalog::Instance(MapCatalog* catalog) {
  _instance = catalog;
}
