/*
 * Copyright (c) 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef projcmds_h
#define projcmds_h

#include <Unidraw/Commands/command.h>
#include <Unidraw/Tools/grcomptool.h>
#include <MapUnidraw/mapcmds.h>

class OvImportCmd;

class ProjShowCmd : public Command {
public:
    ProjShowCmd(ControlInfo*);
    ProjShowCmd(Editor* = nil);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
};

#ifdef HAVE_ACE

// Base class for commands that connect to a server, send a command string
// and import the result

class ProjImportByServCmd : public Command {
public:
    ProjImportByServCmd(ControlInfo*);
    ProjImportByServCmd(Editor* = nil);

    void ServHost(const char* host);
    void ServPort(unsigned short port);
    void ServCom(char* com);
    void ResultImportCmd(OvImportCmd* impcmd);

    virtual void Execute();
    virtual boolean Reversible();

    virtual Command* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
    unsigned short _port;
    char* _host;
    char* _com;
    OvImportCmd* _impcmd;
};

// Sample derived class that imports a map fclass from mapserv

class ProjMapImportByServCmd : public ProjImportByServCmd {
public:
    ProjMapImportByServCmd(ControlInfo*);
    ProjMapImportByServCmd(Editor* = nil);
    virtual void Execute();
    virtual Command* Copy();
};

#endif

class ProjPosTool : public GraphicCompTool {
public:
    ProjPosTool(ControlInfo* = nil);

    virtual Command* InterpretManipulator(Manipulator*);

    virtual Tool* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);
protected:
};

#endif
